@@ -92,6 +92,11 @@ class Find:
         self.query.add_condition(self.lastANDOR, field, 'LIKE', '%' + value + '%')
         return self
 
+    def and_where_not_contains(self, field, value):
+        self.lastANDOR = 'AND'
+        self.query.add_condition(self.lastANDOR, field, 'NOT LIKE', '%' + value + '%')
+        return self
+
     def and_where_in(self, field, value):
         self.lastANDOR = 'AND'
         self.query.add_condition(self.lastANDOR, field, 'IN', value)
@@ -157,6 +162,11 @@ class Find:
         self.query.add_condition(self.lastANDOR, field, 'LIKE', '%' + value + '%')
         return self
 
+    def or_where_not_contains(self, field, value):
+        self.lastANDOR = 'AND' if (self.lastANDOR is None) else 'OR'
+        self.query.add_condition(self.lastANDOR, field, 'NOT LIKE', '%' + value + '%')
+        return self
+
     def or_where_in(self, field, value):
         self.lastANDOR = 'AND' if (self.lastANDOR is None) else 'OR'
         self.query.add_condition(self.lastANDOR, field, 'IN', value)
@@ -200,11 +210,11 @@ class Find:
     async def __then(self, query_compiled):
         timeout = aiohttp.ClientTimeout(total=10*60, connect=None,
                       sock_connect=None, sock_read=None)
+        print("sent")
         async with aiohttp.ClientSession( timeout=timeout) as session:
             async with session.post(
                     self.sbx_core.p(self.url), json=query_compiled,
                     headers=self.sbx_core.get_headers_json()) as resp:
-                print("sent")
                 r = await resp.json()
                 print("response")
                 return r
@@ -216,6 +226,17 @@ class Find:
         self.set_url(True)
         return await self.__then(self.query.compile())
 
+    async def delete(self, page_size=1000, max_in_parallel=2):
+        self.set_url(False)
+        if "keys" in self.query.q['where']:
+            return await self.__then(self.query.compile())
+        else:
+            response = await self.find_all_query(page_size=page_size, max_in_parallel=max_in_parallel)
+            self.where_with_keys([result["_KEY"] for resp in response for result in resp["results"]])
+            return await self.__then(self.query.compile())
+
+
+
     async def gather_with_concurrency(self, n, *tasks):
         semaphore = asyncio.Semaphore(n)
 
@@ -229,28 +250,49 @@ class Find:
     def find_callback(self, callback):
         self.sbx_core.make_callback(self.find(), callback)
 
+    def merge_results(self, results):
+        total_res = {
+            "success": True,
+            "results": [],
+            "fetched_results": {},
+        }
+        for res in results:
+            if res["success"]:
+                total_res["results"] =  total_res["results"] + res["results"]
+                if "fetched_results" in res:
+                    for k,v in res["fetched_results"].items():
+                        if k not in total_res["fetched_results"]:
+                            total_res["fetched_results"][k] = {}
+                        total_res["fetched_results"][k].update(v)
+            else:
+                total_res["success"] = False
+                total_res["message"] = res["message"]
+                break
+        return total_res
+
     async def find_all_query(self, page_size=1000, max_in_parallel=2):
         self.set_page_size(page_size)
+        self.set_page(1)
         self.set_url(True)
         queries = []
         query_compiled = self.query.compile()
         data = await self.__then(query_compiled)
-        total_pages = data['total_pages']
-        for i in range(total_pages+1):
-            query_aux = copy.deepcopy(query_compiled)
-            query_aux['page'] = (i + 1)
-            queries.append(self.__then(query_aux))
-        # futures = self.__chunk_it(queries, min(max_in_parallel, len(queries)))
-        # results = await asyncio.gather(*[futures[i] for i in range(len(futures))])
-        # data = []
-        # for i in range(len(results)):
-        #     for j in range(len(results[i])):
-        #         data.extend(results[i][j])
-        data = []
-        results = await  self.gather_with_concurrency(max_in_parallel, *queries)
-        for i in range(len(results)):
-            data.append(results[i])
-        return data
+        all_data = [data]
+        if data["success"]:
+            total_pages = data['total_pages']
+            for i in range(1, total_pages):
+                query_aux = copy.deepcopy(query_compiled)
+                query_aux['page'] = (i + 1)
+                queries.append(self.__then(query_aux))
+
+            results = await  self.gather_with_concurrency(max_in_parallel, *queries)
+            for i in range(len(results)):
+                all_data.append(results[i])
+        return all_data
+
+
+    async def find_all(self, page_size=1000, max_in_parallel=2):
+        return self.merge_results(await self.find_all_query(page_size, max_in_parallel))
 
     def find_all_callback(self, callback):
         self.sbx_core.make_callback(self.find_all_query(), callback)
@@ -351,7 +393,7 @@ class SbxCore:
         'row': '/data/v1/row',
         'find': '/data/v1/row/find',
         'update': '/data/v1/row/update',
-        'delete': '/data/v1/row/',
+        'delete': '/data/v1/row/delete',
         'downloadFile': '/content/v1/download',
         'uploadFile': '/content/v1/upload',
         'addFolder': '/content/v1/folder',
@@ -362,7 +404,8 @@ class SbxCore:
         'payment_token': '/payment/v1/token',
         'password': '/user/v1/password/request',
         'cloudscript_run': '/cloudscript/v1/run',
-        'domain': '/data/v1/row/model/list'
+        'domain': '/data/v1/row/model/list',
+        'config': '/domain/v1/list/app'
     }
 
     def __init__(self, manage_loop=False):
@@ -411,10 +454,10 @@ class SbxCore:
         sw = False
         if isinstance(data, list):
             for item in data:
-                if "_KEY" in item:
+                if "_KEY" in item and item["_KEY"] is not None:
                     sw = True
         else:
-            if "_KEY" in data:
+            if "_KEY" in data and data["_KEY"] is not None:
                 sw = True
         return sw
 
@@ -448,6 +491,11 @@ class SbxCore:
             async with session.get(self.p(self.urls['domain']), params=params, headers=self.get_headers_json()) as resp:
                 return await resp.json()
 
+    async def get_config(self):
+        async with aiohttp.ClientSession() as session:
+            async with session.get(self.p(self.urls['config']), params={}, headers=self.get_headers_json()) as resp:
+                return await resp.json()
+
     async def run(self, key, params, url=None):
         async with aiohttp.ClientSession() as session:
             params = {'key': key, 'params': params}
@@ -517,6 +565,21 @@ class EventQuery:
                     headers=self.sbx_event.get_headers_json()) as resp:
                 return await resp.json()
 
+    async def then_v2(self, params, body):
+        timeout = aiohttp.ClientTimeout(total=10 * 60, connect=None,
+                                        sock_connect=None, sock_read=None)
+        async with aiohttp.ClientSession(timeout=timeout) as session:
+            if body is None:
+                async with session.get(
+                        self.sbx_event.p(self.url), params=params,
+                        headers=self.sbx_event.get_headers_json()) as resp:
+                    return await resp.json()
+            else:
+                async with session.post(
+                        self.sbx_event.p(self.url), params=params, json=body,
+                        headers=self.sbx_event.get_headers_json()) as resp:
+                    return await resp.json()
+
     def __chunk_it(self, seq, num):
         avg = len(seq) / float(num)
         out = []
@@ -536,6 +599,7 @@ class EventQuery:
 
         responses = await  asyncio.gather(*(sem_task(task) for task in tasks))
         return responses
+
     async def get_all(self, params, division, max_in_parallel=2):
         tempParams = { "toDate":   params["toDate"], "fromDate": params["fromDate"]}
         queries = []
@@ -577,6 +641,50 @@ class EventQuery:
         return {"items": data}
 
 
+    async def get_all_v2(self, params, body, division, max_in_parallel=2):
+        if body is None:
+            return await self.get_all(params,division,max_in_parallel)
+
+        body_temp = copy.copy(body)
+        queries = []
+
+        N = division
+        test_date1 = datetime.datetime.fromisoformat(body_temp["from_date"][:-1])
+        test_date2 = datetime.datetime.fromisoformat(body_temp["to_date"][:-1])
+        temp = []
+
+        diff = (test_date2 - test_date1) // N
+        for idx in range(0, N+1):
+            temp.append((test_date1 + idx * diff))
+
+        dates = []
+        for sub in temp:
+            dates.append(sub.strftime("%Y-%m-%dT%H:%M:%S.000Z"))
+
+        print(N)
+        print(dates)
+        for i in range(len(dates)-1):
+            body_temp["from_date"] = dates[i]
+            body_temp["to_date"] = dates[i+1]
+            queries.append(self.then_v2(params, body_temp))
+            body_temp = copy.copy(body_temp)
+
+        # futures = self.__chunk_it(queries, min(max_in_parallel, len(queries)))
+        # results = await asyncio.gather(*[futures[i] for i in range(len(futures))])
+        # data = []
+        # for i in range(len(results)):
+        #     for j in range(len(results[i])):
+        #         if "items" in results[i][j]:
+        #             data.extend(results[i][j]["items"])
+        data = []
+        results = await self.gather_with_concurrency(max_in_parallel, *queries)
+        for i in range(len(results)):
+            if "items" in results[i]:
+                data.extend(results[i]["items"])
+
+        return {"items": data}
+
+
 class SbxEvent:
     '''
         This is the core of the communication with SbxEvent.
@@ -660,25 +768,34 @@ class WorkflowQuery:
         responses = await  asyncio.gather(*(sem_task(task) for task in tasks))
         return responses
 
-    async def then(self, params):
-        async with aiohttp.ClientSession() as session:
+    async def then(self, params, method="get"):
+        timeout = aiohttp.ClientTimeout(total=10 * 60, connect=None,
+                                        sock_connect=None, sock_read=None)
+        async with aiohttp.ClientSession(timeout=timeout) as session:
+            if method == "post":
+                async with session.post(
+                        self.sbx_workflow.p(self.url), json=params,
+                        headers=self.sbx_workflow.get_headers_json()) as resp:
+                    return await resp.json()
+
             async with session.get(
                     self.sbx_workflow.p(self.url), params=params,
                     headers=self.sbx_workflow.get_headers_json()) as resp:
                 return await resp.json()
 
-    async def find_all(self, params, page_size=1000, max_in_parallel=2):
+    async def find_all(self, params, page_size=1000, max_in_parallel=2, method="get"):
 
         params['size'] = page_size
         params['page'] = 0
         queries = []
-        data = []
-        temp = await self.then(params)
+
+        temp = await self.then(params, method)
         total_pages = math.ceil(temp['totalCount']/page_size)
-        for i in range(total_pages):
+        data = [temp]
+        for i in range(1, total_pages):
             params_aux = copy.deepcopy(params)
             params_aux['page'] = i
-            queries.append(self.then(params_aux))
+            queries.append(self.then(params_aux, method))
         if len(queries) > 0:
             # futures = self.__chunk_it(queries, min(max_in_parallel, len(queries)))
             # results = await asyncio.gather(*[futures[i] for i in range(len(futures))])
@@ -688,8 +805,6 @@ class WorkflowQuery:
             results = await  self.gather_with_concurrency(max_in_parallel, *queries)
             for i in range(len(results)):
                 data.append(results[i])
-        else:
-            data.append(temp)
         return data
 
 
